package se.chalmers.cs.gf.graph;

import se.gu.ling.trindikit.oaa.common.*;

import com.sri.oaa2.icl.*;

import se.chalmers.cs.gf.dialogutil.IclUtil;

import java.util.List;

/**
 *  Example solution:
 *  shortest_path('Valand','Chalmers',[leg('6','Valand','Chalmers', 3.0)])
 */
public class ShortestPathSolver extends OAASolver {

        private static String solvableString = "shortest_path(From,To,Path)";

        private DijkstraPath pathFinder;

        public ShortestPathSolver(Graph<?,String> graph) {
                super(solvableString);
                this.pathFinder = new DijkstraPath(graph);
        }

        public boolean solve(IclTerm goal, IclList params, IclList answers) {
                if (!(goal.getTerm(0).isStr() && goal.getTerm(1).isStr()))
                        return false;

                IclStr fromTerm = (IclStr)goal.getTerm(0);
                IclStr toTerm = (IclStr)goal.getTerm(1);

                String from = fromTerm.toUnquotedString();
                String to = toTerm.toUnquotedString();

                Path path = pathFinder.computePath(from, to);
                IclTerm pathTerm = pathToTerm(path);
                answers.add(new IclStruct("shortest_path", 
                                          fromTerm, toTerm, 
                                          pathTerm));

                return true;
        }

        private IclTerm pathToTerm(Path path) {
                IclList t = new IclList();
                for (Path.Leg l : path.getLegs())
                        t.add(legToTerm(l));
                return t;
        }

        private IclTerm legToTerm(Path.Leg l) {
                return new IclStruct("leg", 
                                     new IclStr(l.getLabel()), 
                                     new IclStr(l.getFrom()),
                                     new IclStr(l.getTo()),
                                     new IclFloat(l.getDist()));
        }

}
